/*
 * Copyright (c) 1992 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: filter.cc,v 1.3 92/06/08 02:55:43 van Exp $ (LBL)";
#endif

extern "C" int bpf_init(const char *ifname, const char *filter,
			int *bufsize, int snap);

#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <net/bpf.h>
#include "filter.h"
#include <osfcn.h>

PacketFilter::PacketFilter(const char* device, const char* cmdbuf)
{
	fd = bpf_init(device, cmdbuf, &bufsize, 4);
	if (fd < 0) {
		perror("bpf");
		exit(1);
	}
	u_int imm = 1;
	if (ioctl(fd, BIOCIMMEDIATE, (char *)&imm) < 0) {
		perror("bpf: BIOCIMMEDIATE");
		exit(1);
	}
	buffer = new char[bufsize];
}

int PacketFilter::ReadLen()
{
	int nr = read(fd, buffer, bufsize);
	int len = 0;
	char* bp = buffer;
	char* ep = bp + nr;
	while (bp < ep) {
		bpf_hdr* bhp = (bpf_hdr*)bp;
		len += bhp->bh_datalen;
		bp += BPF_WORDALIGN(bhp->bh_hdrlen + bhp->bh_caplen);
	}
	return (len);
}

u_int PacketFilter::ReadDrops()
{
	struct bpf_stat bs;
	if (ioctl(fd, BIOCGSTATS, (char *)&bs) < 0) {
		perror("bpf: BIOCGSTATS");
		exit(1);
	}
	return (bs.bs_drop);
}
