/*
 * Copyright (c) 1991 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static char rcsid[] =
    "@(#) $Header: numbox.cc,v 1.3 92/07/31 11:51:29 van Exp $ (LBL)";
#endif

#include <ctype.h>

#include <osfcn.h>
#undef resource_h

#include <InterViews/painter.h>
#include <InterViews/sensor.h>
#include <InterViews/event.h>
#include <InterViews/color.h>
#include <InterViews/font.h>
#include <InterViews/shape.h>
#include <InterViews/strtable.h>

#include "numbox.h"

static const int inset = 2;

NumBox::~NumBox()
{
	Unref(font);
}

NumBox::NumBox(int w)
{
	Init(stdfont, w);
}

void NumBox::Init(Font *fn, int w)
{
	width = w;
	font = fn;
	font->Reference();

	value = 0.0;
	selected = 0;
	editing = 0;
	highlighted = 0;
	keyindex = 0;
	justify = Center;

	input = new Sensor;
	input->Reference();
	input->Catch(KeyEvent);
	input->Catch(EnterEvent);
	input->Catch(LeaveEvent);
}

void NumBox::Reconfig()
{
	shape->Rigid();
	shape->width = width;
	shape->height = font->Height() + 2 * inset;
	if (width == 0)
		shape->hstretch = hfil;

	Unref(output);
	output = new Painter;
	output->Reference();
	output->SetFont(font);
}

void NumBox::DoSelect(int val)
{
	selected = val;
	Draw();
}

void NumBox::Redraw(Coord x0, Coord y0, Coord x1, Coord y1)
{
	char buf[80];
	if (selected)
		output->SetColors(white, black);
	else
		output->SetColors(black, white);

	output->ClearRect(canvas, 0, 0, xmax, ymax);
	if (highlighted)
		output->Rect(canvas, 1, 1, xmax - 1, ymax - 1);

	int left;
	char *s;
	if (editing) {
		s = keybuf;
		left = inset;
	} else {
		sprintf(buf, "%g", value);
		s = buf;
		int width = font->Width(s);
		switch (justify) {

			/* Center */
		default:
			left = width > xmax ? inset : (xmax - width) / 2;
			break;

		case Left:
			left = inset + 1;
			break;

		case Right:
			left = xmax - width - inset;
			break;
		}
	}
	output->Text(canvas, s, left, inset);
}

#include <InterViews/strtable.h>

void NumBox::Handle(Event &e)
{
	switch (e.eventType) {

	case KeyEvent:
		DoKey(e.keystring[0]);
		break;

	case EnterEvent:
		highlighted = 1;
		Draw();
		break;

	case LeaveEvent:
		highlighted = 0;
		editing = 0;
		Draw();
		break;
	}
}

void NumBox::DoKey(int c)
{		
	if (c == 27) {
		editing = 0;
		Draw();
		return;
	}
	if (c == 13) {
		if (!editing)
			return;
		editing = 0;
		(void)sscanf(keybuf, "%lf", &value);
		Update();
		Draw();
		return;
	}
	if (c == 8) {
		if (keyindex > 0) {
			keybuf[--keyindex] = 0;
			Draw();
		}
		return;
	}
	// XXX isprint?
	if (1) {
		if (editing == 0) {
			editing = 1;
			keyindex = 0;
		}
		if (keyindex < int(sizeof(keybuf)) - 1) {
			keybuf[keyindex++] = c;
			keybuf[keyindex] = 0;
			Draw();
		}
	}
}
