/*
 * Copyright (c) 1992 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: screen.cc,v 1.4 92/06/08 02:55:46 van Exp $ (LBL)";
#endif

#include <osfcn.h>
#undef resource_h

#include <InterViews/sensor.h>
#include <InterViews/frame.h>

#include "screen.h"
#include "filter.h"
#include "stripchart.h"


static inline double TimeOfDay()
{
	timeval tv;
	gettimeofday(&tv, 0);
	return (double(tv.tv_sec) + double(tv.tv_usec)*1e-6);
}

Screen::Screen(PacketFilter* f, StripChart* c, double p, double G, double S)
	: filter(f), chart(c), period(p), gain(G), scale(S)
{
	sum = 0;
	curval = 0;

	input = new Sensor;
	input->Reference();
	input->Catch(KeyEvent);
	input->CatchChannel(filter->Chan());
	/* If we make this too big, interviews starves us of timerevents */
	input->CatchTimer(0, 20000);

	Insert(chart);

	timebase = TimeOfDay();
	nexttime = timebase + period;
}

void Screen::Advance()
{
	double now = TimeOfDay();

	if (now >= nexttime) {
		/*
		 * 'lasttime' is the last time we zeroed sum.  The
		 * current value of sum is the amount of data we have
		 * seen since then so sum/(now-lasttime) is the average
		 * data rate since then.  'Nexttime' is the next time
		 * we should add a point to the chart.  If we got blocked
		 * for some reason, now may be several periods beyond
		 * this so add points until we catch up ('avgrate' is
		 * the correct average rate for all these periods).
		 */
		double deltat = now - lasttime;
		double s = double(sum);
		lasttime = now;
		sum = 0;
		if (gain < 1.)
			curval = s = curval + gain * (s - curval);
		double avgrate = s / (deltat * scale);
		while (nexttime <= now) {
			chart->NewPoint(nexttime, avgrate);
			nexttime += period;
		}
	}
}

void Screen::Handle(Event& e)
{
	switch(e.eventType) {

	case ChannelEvent:
		if (e.channel & filter->IOMask()) {
			sum += filter->ReadLen();
			Advance();
		}
		break;

	case TimerEvent:
		Advance();
		break;

	case KeyEvent:
		switch (*e.keystring) {
		case 3:
		case 4:
		case 'q':
		case 'Q':
			/* XXX */
			exit(0);
			/* NOTREACHED */

		case 's':
		case 'S':
			printf("%d drops\n", filter->ReadDrops());
			break;
		}
		break;

	default:
		break;
	}
}
