/*
 * Copyright (c) 1991 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: stripchart.cc,v 1.3 92/06/08 02:55:50 van Exp $ (LBL)";
#endif

#include <osfcn.h>
#undef resource_h
#include <math.h>

#include <InterViews/box.h>
#include <InterViews/painter.h>
#include <InterViews/pattern.h>
#include <InterViews/font.h>
#include <InterViews/shape.h>
#include <InterViews/frame.h>
#include <InterViews/message.h>
#include <InterViews/glue.h>
#include <InterViews/border.h>
#include <InterViews/canvas.h>

#include "numbox.h"
#include "stripchart.h"


/*
 * XXX Use IV StringEditor instead of blasted NumBox.
 */
class StripChartDim : public NumBox {
public:
	StripChartDim(double, StripChart*);
	inline double Get() { return value; }
	void Update() { sc->UpdateDim(); }
private:
	StripChart *sc;
};

StripChartDim::StripChartDim(double v, StripChart *p)
{
	value = v;
	sc = p;
	SetJustify(Right);
}

class StripPlot : public Interactor {
    public:
	StripPlot(StripChart* c) : chart(c) { }
	virtual void Redraw(Coord, Coord, Coord, Coord);
	virtual void Resize();
    protected:
	StripChart* const chart;
};

void StripPlot::Resize()
{
	chart->Rescale();
}

void StripPlot::Redraw(Coord x0, Coord y0, Coord x1, Coord y1)
{
	chart->Redraw(x0, y0, x1, y1);
}

static inline int RoundUp(int n)
{
	for (int k = 1; n != 0; n >>= 1, k <<= 1)
		;
	return k;
}

void StripChart::Rescale()
{
	Canvas* pcanvas = plot->GetCanvas();
	Coord xm = pcanvas->Width() - 1;
	Coord ym = pcanvas->Height() - 1;
	xscale = double(xm) / width->Get();
	yscale = double(ym) / height->Get();
}

#define ROUND(x) int((x) + 0.5)

void StripChart::NewPoint(double x, double y)
{
	Point *p = &pts[++curpt & index_mask];
	p->x = x;
	p->y = y;

	double w = width->Get();
	double dx = x - offset;
	if (dx < 0 || dx >= 2. * w) {
		offset = x - w;
		Draw();
		return;
	}
	Canvas* pcanvas = plot->GetCanvas();
	Coord xm = pcanvas->Width() - 1;
	Coord cx = ROUND((x - offset) * xscale);
	int dw = cx - xm;
	if (dw > 0) {
		offset = x - w;
		Coord lx = xm - dw + 1;
		Coord ym = pcanvas->Height() - 1;
		output->Copy(pcanvas, dw, 0, xm, ym, pcanvas, 0, 0);
		output->ClearRect(pcanvas, lx, 0, xm, ym);

		double g;
		if ((g = gridy) != 0.) {
			double h = height->Get();
			for (double dy = g; dy < h; dy += g) {
				Coord ly = ROUND(dy * yscale);
				output->Line(pcanvas, lx, ly, xm, ly);
			}
		}
		if ((g = gridx) != 0. && x >= nextgridx) {
			if (nextgridx == 0.)
				nextgridx = aint(x / g) * g;
			for ( ; nextgridx <= x; nextgridx += g) {
				Coord gx = ROUND((nextgridx - offset) * xscale);
				output->Line(pcanvas, gx, 0, gx, ym);
			}
		}

		cx = xm;
	}
	Coord cy = ROUND(y * yscale);
	output->Line(pcanvas, cx, 0, cx, cy);
}

void StripChart::Redraw(Coord x0, Coord y0, Coord x1, Coord y1)
{
	Canvas* pcanvas = plot->GetCanvas();
	Coord xm = pcanvas->Width() - 1;
	Coord ym = pcanvas->Height() - 1;
	output->ClearRect(pcanvas, 0, 0, xm, ym);

	int n = Npts() - 1;
	if (n <= 0)
		return;

	double g;
	if ((g = gridy) != 0.) {
		double h = height->Get();
		for (double dy = g; dy < h; dy += g) {
			Coord ly = ROUND(dy * yscale);
			output->Line(pcanvas, 0, ly, xm, ly);
		}
	}
	if ((g = gridx) != 0.) {
		double x = CurrentPoint()->x;
		nextgridx = (aint(x / g) + 1.) * g;
		for (double nx = nextgridx; (nx -= g) > offset; ) {
			Coord gx = ROUND((nx - offset) * xscale);
			output->Line(pcanvas, gx, 0, gx, ym);
		}
	}

	for (int i = 0; i < n; ++i) {
		const Point *p = DataPoint(i);
		Coord cx = ROUND((p->x - offset) * xscale);
		if (cx >= x0 && cx <= x1) {
			Coord cy = ROUND(p->y * yscale);
			output->Line(pcanvas, cx, 0, cx, cy);
		}
	}
}

StripChart::StripChart(const char* label, const double w, const double h,
		       const int gx, const int gy, const int npts)
	: gxdiv(gx), gydiv(gy), maxpts(RoundUp(npts)),
	  index_mask(RoundUp(npts) - 1), pts(new Point[RoundUp(npts)])
{
	width = new StripChartDim(w, this);
	height = new StripChartDim(h, this);
	width->SetJustify(Right);
	height->SetJustify(Left);

	curpt = 0;
	offset = 0;
	xscale = 0.;
	yscale = 0.;

	gridx = gx? w / gx : 0.;
	gridy = gy? h / gy : 0.;

	SetName("StripChart");

	HBox *hbox = new HBox(new Message(label, Left, 2, 0, 0),
			      new HGlue, width,
			      new Message("x"), height);
	plot = new StripPlot(this);
	VBox* vbox = new VBox(hbox, new HBorder, plot);
	Insert(new Frame(vbox));
}

StripChart::~StripChart()
{
	delete plot;
	delete pts;
	delete width;
	delete height;
}

void StripChart::UpdateDim()
{
	/*
	 * Adjust the offset so we such as much data as possible, but also
	 * so that graphs with the same widths will line up.
	 */
	if (Npts() > 0 && CurrentPoint()->x > width->Get())
		offset = CurrentPoint()->x - width->Get();
	else
		offset = 0;
	if (gxdiv)
		gridx = width->Get() / gxdiv;
	if (gydiv)
		gridy = height->Get() / gydiv;
	Rescale();
	Draw();
}

void StripChart::Reset()
{
	offset = 0;
	curpt = 0;
	Draw();
}
