/*
 * Copyright (c) 1991 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Lawrence Berkeley Laboratory,
 * Berkeley, CA.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef viewnet_stripchart_h
#define viewnet_stripchart_h

class StripChartDim;
class StripChart;

struct Point {
	double x;
	double y;
};

class StripChart : public MonoScene {
public:
	StripChart(const char* label, double w, double h,
		   int gridx = 0, int gridy = 0, int npts = 1024);
	~StripChart();

	virtual void Redraw(Coord, Coord, Coord, Coord);

	void Reset();
	void Rescale();
	void UpdateDim();
	void NewPoint(double, double);

protected:
	StripChartDim* GetHeight() const { return height; }
	StripChartDim* GetWidth() const { return width; }

	inline int Npts() const { return min(curpt, maxpts); }

	/* The ith newest data point.  They get drawn from right to left. */
	inline const Point* DataPoint(int i) const
		{ return (const Point *)&pts[(curpt - i) & index_mask]; }
	inline const Point* CurrentPoint() const
		{ return (const Point *)&pts[curpt & index_mask]; }

	Interactor* plot;
	const int index_mask;
	int curpt;
	const int maxpts;
	Point* const pts;

	double gridx;
	double gridy;
	double nextgridx;
	const int gxdiv;
	const int gydiv;

	StripChartDim* width;
	StripChartDim* height;

	double xscale;
	double yscale;

	/* Current offset into data space (i.e. x coord of left part of win) */
	double offset;
};

#endif
